//----------------------------------------------------------------------------------------------
// BSP : Seeed nRF52 Borads 1.1.8
// Board : Seeed nRF52 Borads / Seeed XIAO nRF52840 Sense 
//----------------------------------------------------------------------------------------------
// 2024/09/19 @msfujino

#include <Adafruit_SPIFlash.h>    // 4.3.4, NOTE:need to deleted /Documents/Arduino/libraries/SdFat
#include <flash_devices.h>

Adafruit_FlashTransport_QSPI flashTransport;
Adafruit_SPIFlash flash(&flashTransport);
static const SPIFlash_Device_t my_flash_devices[] = {P25Q16H,};   // On board Flash
const int flashDevices = 1;

#define PIN_WAKEUP D1    // Wakeup gpio pin
#define SLEEP_TIME 20    // Sleep time mS

bool intFlag = false;    // Interrupt flag

void setup()
{
  // LED setting
  pinMode(LED_RED, OUTPUT);
  digitalWrite(LED_RED, HIGH);
  pinMode(LED_GREEN, OUTPUT);
  digitalWrite(LED_GREEN, HIGH);

  // Wakeup pin setting
  pinMode(PIN_WAKEUP, INPUT_PULLUP);
  attachInterrupt(digitalPinToInterrupt(PIN_WAKEUP), interruptCallback, FALLING);
  noInterrupts();

  // Enable DC-DC converter Mode
  NRF_POWER->DCDCEN = 1;            // Enable DC/DC converter for REG1 stage

  // Intitial state
  delay(1000);
  
  // On board Flash enter to Deep Power-Down Mode
  flash.begin(my_flash_devices, flashDevices);
  flashTransport.begin();
  flashTransport.runCommand(0xB9);  // Enter deep power-down mode
  delayMicroseconds(5);             // tDP=3uS
  flashTransport.end();
  
  // indicate end of initialization
  digitalWrite(LED_GREEN, LOW);
  delay(1000);
  digitalWrite(LED_GREEN, HIGH);

  // Light Sleep Mode (RTOS delay() function)
  delay(3000);

  // Enable interrupt
  interrupts();
}

void loop()
{
  // If interrupt occur
  if(intFlag == true) {
    digitalWrite(LED_GREEN, LOW);
    delay(2000);
    digitalWrite(LED_GREEN, HIGH);
    intFlag = false;
  }

  // Sleep again
  delay(SLEEP_TIME);
}

// Interrupt callback function
void interruptCallback()
{
  digitalWrite(LED_RED, LOW);   // Indicate interrupt
  delayMicroseconds(10000);     // delay() cannot be used in interrupt handling
  digitalWrite(LED_RED, HIGH); 

  intFlag = true;               // Set interrupt flag
}
